

/* Epydoc CSS Stylesheet for pyglet
 *
 * This stylesheet can be used to customize the appearance of epydoc's
 * HTML output.
 *
 */

/* Default Colors & Styles
 *   - Set the default foreground & background color with 'body'; and 
 *     link colors with 'a:link' and 'a:visited'.
 *   - Use bold for decision list terms.
 *   - The heading styles defined here are used for headings *within*
 *     docstring descriptions.  All headings used by epydoc itself use
 *     either class='epydoc' or class='toc' (CSS styles for both
 *     defined below).
 */
body                        { background: #ffffff; color: #000000; }
dt                          { font-weight: normal; }
 
/* Page Header & Footer
 *   - The standard page header consists of a navigation bar (with
 *     pointers to standard pages such as 'home' and 'trees'); a
 *     breadcrumbs list, which can be used to navigate to containing
 *     classes or modules; options links, to show/hide private
 *     variables and to show/hide frames; and a page title (using
 *     <h1>).  The page title may be followed by a link to the
 *     corresponding source code (using 'span.codelink').
 *   - The footer consists of a navigation bar, a timestamp, and a
 *     pointer to epydoc's homepage.
 */ 

table.navbar {
    border-top: 1px solid silver;
    border-bottom: 2px solid silver;
    padding: 0px;
    font-family: sans-serif;
    font-size: 80%;
    width: 100%;
}
table.navbar th  {
    padding-right: 1em;
    font-weight: normal;
}
table.navbar th.last {
    text-align: right;
    padding-right: 0px;
}
table.navbar a { 
    text-decoration: none; 
}
th.navbar-breadcrumbs {
    text-align: center;
    width: 100%;
}

/*
.breadcrumbs {
    font-family: sans-serif;
}
*/

h1.epydoc {
    font-family: Arial, sans-serif;
    font-size: 20pt;
    font-weight: normal;
}

h1 {
    font-family: Aria, sans-serif;
    font-size: 16pt;
    font-weight: normal;
}

h2 {
    font-size: 16pt;
    color: #ff6161;
    font-weight: normal;
    font-family: sans-serif;
}

h3 {
    margin-bottom: .5em;
}


/* Frames-based table of contents headers
 *   - Consists of two frames: one for selecting modules; and
 *     the other listing the contents of the selected module.
 *   - h1.toc is used for each frame's heading
 *   - h2.toc is used for subheadings within each frame.
 */
h1.toc {
    font-size: 105%;
    font-weight: bold;
    font-family: sans-serif;
    margin: 0; 
    padding: 0; 
}
h2.toc {
    font-size: 100%;  
    color: black;
    margin-bottom: 0.5em;
    border-bottom: 1px solid black;
}


span.options                { font-size: 70%; }
span.codelink               { font-size: 85%; }
td.footer                   { font-size: 85%; }


th.group-header             { text-align: left; font-style: italic; 
                              font-size: 115%; 
                              border: 1px solid #608090; }


table.summary               { border-collapse: collapse;
                              border: 1px solid silver;
                              margin-bottom: 0.5em;
                              width: 100%;}
.summary td { 
    border: 1px solid silver; 
    padding: 5px;
}
code.summary-type           { font-size: 85%; }
span.inherited-from         { font-size: 85%; }


/* Fields */
dl.fields                   { margin-left: 0px; margin-top: 1em;
                              margin-bottom: 1em; }
dl.fields > dt              { font-weight: bold; }
dl.fields > dd              { margin-bottom: 1em; }
div.fields                  { }
div.fields p                { margin-bottom: 0.5em; }

/* Index tables (identifier index, term index, etc)
 *   - link-index is used for indices containing lists of links
 *     (namely, the identifier index & term index).
 *   - index-where is used in link indices for the text indicating
 *     the container/source for each link.
 *   - metadata-index is used for indices containing metadata
 *     extracted from fields (namely, the bug index & todo index).
 */
table.link-index            { border-collapse: collapse;
                              border: 1px solid silver; }
td.link-index               { border-width: 0px; }
span.index-where            { font-size: 70%; }
table.metadata-index        { border-collapse: collapse;
                              border: 1px solid #608090; 
                              margin: .2em 0 0 0; }
td.metadata-index           { border-width: 1px; border-style: solid; }
.link-index a {
    text-decoration: none;
}


/* Summaries */

.sig-name                   {  }
.sig-arg                    {  }
.sig-default                {  }
.summary-sig                { font-family: monospace; }
.summary-sig-name           {  font-weight: bold; }
a.summary-sig-name {  
    text-decoration: none;
}

.summary-name {
    font-family: monospace;
    font-weight: bold;
    text-decoration: none;
}

.summary-sig-arg            {  }
.summary-sig-default        {  }
.summary-type               { font-family: monospace; }
.summary-description        { margin-left: 1em; }

/* Details */

.fspecial {
    font-family: serif;
    font-style: italic;
}

.detail {
    border-top: 1px solid silver;
}

.detail-signature {
    font-family: monospace;
    margin-top: .5em;
    margin-bottom: .5em;
}

.detail-description {
    margin-top: .5em;
    margin-left: 2em;
}

.pname {
    font-family: monospace;
    font-weight: bold;
}

.ptype {
    font-family: monospace;
}

.ptype a {
    text-decoration: none;
}

.rtype, .overrides {
    font-family: monospace;
}

.rtype a, .overrides a {
    text-decoration: none;
}


/* To render variables, classes etc. like functions */
/*
table.summary .summary-name { font-weight: bold;
                              font-family: monospace; }
table.summary
     a.summary-name:link    { font-weight: bold;
                              font-family: monospace; }
table.summary
    a.summary-name:visited  { font-weight: bold;
                              font-family: monospace; }*/

/* Variable values
 *   - In the 'variable details' sections, each varaible's value is
 *     listed in a 'pre.variable' box.  The width of this box is
 *     restricted to 80 chars; if the value's repr is longer than
 *     this it will be wrapped, using a backslash marked with
 *     class 'variable-linewrap'.  If the value's repr is longer
 *     than 3 lines, the rest will be ellided; and an ellipsis
 *     marker ('...' marked with 'variable-ellipsis') will be used.
 *   - If the value is a string, its quote marks will be marked
 *     with 'variable-quote'.
 *   - If the variable is a regexp, it is syntax-highlighted using
 *     the re* CSS classes.
 */
.variable-linewrap          { color: #604000; font-weight: bold; }
.variable-ellipsis          { color: #604000; font-weight: bold; }
.variable-quote             { color: #604000; font-weight: bold; }
.variable-group             { color: #008000; font-weight: bold; }
.variable-op                { color: #604000; font-weight: bold; }
.variable-string            { color: #006030; }
.variable-unknown           { color: #a00000; font-weight: bold; }
.re                         { color: #000000; }
.re-char                    { color: #006030; }
.re-op                      { color: #600000; }
.re-group                   { color: #003060; }
.re-ref                     { color: #404040; }

/* Base tree
 *   - Used by class pages to display the base class hierarchy.
 */
pre.base-tree               { font-size: 80%; margin: 0; }


/*---------------------------------------------------------------------
  Custom epydoc source coloring
  ---------------------------------------------------------------------*/
pre.py-doctest, pre.variable
                            { padding: .5em; margin: 1em;
                              color: #000000;
                              border-left: 2px solid silver; }
pre.py-src                  { border: 2px solid #000000; 
                              background: #f0f0f0; color: #000000; }
.py-line                    { border-left: 2px solid #000000; 
                              margin-left: .2em; padding-left: .4em; }
.py-lineno                  { font-style: italic; font-size: 90%;
                              padding-left: .5em; }
a.py-toggle                 { text-decoration: none; }
div.py-highlight-hdr        { border-top: 2px solid #000000;
                              border-bottom: 2px solid #000000;
                              background: #d8e8e8; }
/*div.py-highlight            { border-bottom: 2px solid #000000;
                              background: #d0e0e0; }*/
.py-prompt                  { color: #005050; font-weight: bold;}
.py-more                    { color: #005050; font-weight: bold;}
.py-string                  { color: #006030; }
.py-comment                 { color: #003060; }
.py-keyword                 { color: #600000; }
.py-output                  { color: #404040; }
.py-name                    { color: #000050; }
.py-name:link               { color: #000050 !important; }
.py-name:visited            { color: #000050 !important; }
.py-number                  { color: #005000; }
.py-defname                 { color: #000060; font-weight: bold; }
.py-def-name                { color: #000060; font-weight: bold; }
.py-base-class              { color: #000060; }
.py-param                   { color: #000060; }
.py-docstring               { color: #006030; }
.py-decorator               { color: #804020; }

/* HTML Log */
div.log-block               { padding: 0; margin: .5em 0 .5em 0;
                              background: #e8f0f8; color: #000000;
                              border: 1px solid #000000; }
div.log-error               { padding: .1em .3em .1em .3em; margin: 4px;
                              background: #ffb0b0; color: #000000;
                              border: 1px solid #000000; }
div.log-warning             { padding: .1em .3em .1em .3em; margin: 4px;
                              background: #ffffb0; color: #000000;
                              border: 1px solid #000000; }
div.log-info               { padding: .1em .3em .1em .3em; margin: 4px;
                              background: #b0ffb0; color: #000000;
                              border: 1px solid #000000; }
h2.log-hdr                  { background: #70b0ff; color: #000000;
                              margin: 0; padding: 0em 0.5em 0em 0.5em;
                              border-bottom: 1px solid #000000; font-size: 110%; }
p.log                       { font-weight: bold; margin: .5em 0 .5em 0; }
tr.opt-changed              { color: #000000; font-weight: bold; }
tr.opt-default              { color: #606060; }
pre.log                     { margin: 0; padding: 0; padding-left: 1em; }
