/*---------------------------------------------------------------------
  Start with original docutils stylesheet
  ---------------------------------------------------------------------*/

div.document {
    margin-left: 60px;
    margin-right: 60px;
}

/* used to remove borders from tables and images */
.borderless, table.borderless td, table.borderless th {
  border: 0 }

table.borderless td, table.borderless th {
  /* Override padding for "table.docutils td" with "! important".
     The right padding separates the table cells. */
  padding: 0 0.5em 0 0 ! important }

.first {
  /* Override more specific margin styles with "! important". */
  margin-top: 0 ! important }

.last, .with-subtitle {
  margin-bottom: 0 ! important }

.hidden {
  display: none }

a.toc-backref {
  text-decoration: none ;
  color: inherit;
}

blockquote.epigraph {
  margin: 2em 5em ; }

dl.docutils dd {
  margin-bottom: 0.5em }

/* Uncomment (and remove this text!) to get bold-faced definition list terms
dl.docutils dt {
  font-weight: bold }
*/

div.abstract {
  margin: 2em 5em }

div.abstract p.topic-title {
  font-weight: bold ;
  text-align: center }

div.admonition, div.attention, div.caution, div.danger, div.error,
div.hint, div.important, div.note, div.tip, div.warning {
  margin: 2em ;
  border: medium outset ;
  padding: 1em }

div.admonition p.admonition-title, div.hint p.admonition-title,
div.important p.admonition-title, div.note p.admonition-title,
div.tip p.admonition-title {
  font-weight: bold ;
  font-family: sans-serif }

div.attention p.admonition-title, div.caution p.admonition-title,
div.danger p.admonition-title, div.error p.admonition-title,
div.warning p.admonition-title {
  color: red ;
  font-weight: bold ;
  font-family: sans-serif }

/* Uncomment (and remove this text!) to get reduced vertical space in
   compound paragraphs.
div.compound .compound-first, div.compound .compound-middle {
  margin-bottom: 0.5em }

div.compound .compound-last, div.compound .compound-middle {
  margin-top: 0.5em }
*/

div.dedication {
  margin: 2em 5em ;
  text-align: center ;
  font-style: italic }

div.dedication p.topic-title {
  font-weight: bold ;
  font-style: normal }

div.figure {
  text-align: center;
  margin-left: 2em ;
  margin-right: 2em }

div.footer, div.header {
  clear: both;
  font-size: smaller }

div.line-block {
  display: block ;
  margin-top: 1em ;
  margin-bottom: 1em }

div.line-block div.line-block {
  margin-top: 0 ;
  margin-bottom: 0 ;
  margin-left: 1.5em }

div.sidebar {
  margin-left: 1em ;
  border: medium outset ;
  padding: 1em ;
  background-color: #ffffee ;
  width: 40% ;
  float: right ;
  clear: right }

div.sidebar p.rubric {
  font-family: sans-serif ;
  font-size: medium }

div.system-messages {
  margin: 5em }

div.system-messages h1 {
  color: red }

div.system-message {
  border: medium outset ;
  padding: 1em }

div.system-message p.system-message-title {
  color: red ;
  font-weight: bold }

div.topic {
  margin: 2em }

h1, h2, h3, h4, h5, h6 {
  font-family: Arial, sans-serif;
  margin-top: 1em;
}

h1 {
    font-size: 16pt;
    color: #ff6161;
    font-weight: normal;
}

h1.title {
    color: black;
    font-family: Arial, sans-serif;
    font-size: 24pt;
    margin-top: 96px;
    margin-bottom: 72px;
    text-align: right;
    border-bottom: 1px solid black;
}

hr.docutils {
  width: 75% }

img.align-left {
  clear: left }

img.align-right {
  clear: right }

ol.simple, ul.simple {
  margin-bottom: 1em }

ol.arabic {
  list-style: decimal }

ol.loweralpha {
  list-style: lower-alpha }

ol.upperalpha {
  list-style: upper-alpha }

ol.lowerroman {
  list-style: lower-roman }

ol.upperroman {
  list-style: upper-roman }

p.attribution {
  text-align: right ;
  margin-left: 50% }

p.caption {
  font-style: italic }

p.credits {
  font-style: italic ;
  font-size: smaller }

p.label {
  white-space: nowrap }

p.rubric {
  font-weight: bold ;
  font-size: larger ;
  color: maroon ;
  text-align: center }

p.sidebar-title {
  font-family: sans-serif ;
  font-weight: bold ;
  font-size: larger }

p.sidebar-subtitle {
  font-family: sans-serif ;
  font-weight: bold }

p.topic-title {
  font-weight: bold }

pre.address {
  margin-bottom: 0 ;
  margin-top: 0 ;
  font-family: serif ;
  font-size: 100% }

pre.literal-block, pre.doctest-block {
  margin-left: 2em ;
  margin-right: 2em }

span.classifier {
  font-family: sans-serif ;
  font-style: oblique }

span.classifier-delimiter {
  font-family: sans-serif ;
  font-weight: bold }

span.interpreted {
  font-family: sans-serif }

span.option {
  white-space: nowrap }

span.pre {
  white-space: pre }

span.problematic {
  color: red }

span.section-subtitle {
  /* font-size relative to parent (h1..h6 element) */
  font-size: 80% }

table.citation {
  border-left: solid 1px gray;
  margin-left: 1px }

table.docinfo {
  margin: 2em 4em }

table.footnote {
  border-left: solid 1px black;
  margin-left: 1px }

table.docutils {
  margin-top: 0.5em ;
  margin-bottom: 0.5em;
  border: 1px solid #708890;
  border-collapse: collapse;
}

table.docutils td, table.docutils th,
table.docinfo td, table.docinfo th {
  padding-left: 0.5em ;
  padding-right: 0.5em ;
  vertical-align: top }

table.docutils th.field-name, table.docinfo th.docinfo-name {
  font-weight: bold ;
  text-align: left ;
  white-space: nowrap ;
  padding-left: 0 }

table.docutils th {
  background: #e8f0f8;
}

h1 tt.docutils, h2 tt.docutils, h3 tt.docutils,
h4 tt.docutils, h5 tt.docutils, h6 tt.docutils {
  font-size: 100% }

ul.auto-toc {
  list-style-type: none }

table.images td {
    vertical-align: middle;
    text-align: center;
    padding: 4px;
}

table.images th.stub {
    vertical-align: top;
    text-align: left;
    font-weight: normal;
}

table.images th.head {
    font-weight: bold;
}

.filelink {
    font-family: monospace;
}

/*---------------------------------------------------------------------
  Add epydoc's source coloring
  ---------------------------------------------------------------------*/
pre.py-doctest 
                            { padding: .5em; margin: 1em;
                              color: #000000;
                              border-left: 2px solid silver; }
table pre.py-doctest        { background: #dce4ec;
                              color: #000000; }
pre.py-src                  { border: 2px solid #000000; 
                              background: #f0f0f0; color: #000000; }
.py-line                    { border-left: 2px solid #000000; 
                              margin-left: .2em; padding-left: .4em; }
.py-lineno                  { font-style: italic; font-size: 90%;
                              padding-left: .5em; }
a.py-toggle                 { text-decoration: none; }
div.py-highlight-hdr        { border-top: 2px solid #000000;
                              border-bottom: 2px solid #000000;
                              background: #d8e8e8; }
div.py-highlight            { border-bottom: 2px solid #000000;
                              background: #d0e0e0; }
.py-prompt                  { color: #005050; font-weight: bold;}
.py-more                    { color: #005050; font-weight: bold;}
.py-string                  { color: #006030; }
.py-comment                 { color: #003060; }
.py-keyword                 { color: #600000; }
.py-output                  { color: #404040; }
.py-name                    { color: #000050; }
.py-name:link               { color: #000050 !important; }
.py-name:visited            { color: #000050 !important; }
.py-number                  { color: #005000; }
.py-defname                 { color: #000060; font-weight: bold; }
.py-def-name                { color: #000060; font-weight: bold; }
.py-base-class              { color: #000060; }
.py-param                   { color: #000060; }
.py-docstring               { color: #006030; }
.py-decorator               { color: #804020; }

pre.py-doctest strong       { color: #cc0000; font-weight: bold}
pre.py-doctest strong *     { color: #cc0000; font-weight: bold}

/*---------------------------------------------------------------------
  Page navigation
  ---------------------------------------------------------------------*/
.navigation {
    overflow: clip;
    position: absolute;
    left: 0px;
    right: 0px;
    padding: 2px;
    font-family: sans-serif;
    font-size: 80%;
    text-align: center;
    border-top: 1px solid silver;
    border-bottom: 1px solid silver;
}

.navigation-header {
    top: 5px;
    border-bottom: 2px solid silver;
}

.navigation-footer {
    margin-top: 72px;
    border-top: 2px solid silver;
}

.navigation > .previous {
    float: left;
    margin-right: 1em;
}

.navigation > .next {
    float: right;
    margin-left: 1em;
}

.navigation > .breadcrumbs {
}
